Attribute VB_Name = "MBetter32"
Option Explicit
#If Win32 Then
DefLng A-Z
#Else
DefInt A-Z
#End If

#If Win32 Then

Private Type TFileInfo
    hIcon As Long
    iIcon As Long
    dwAttributes As Long
    szDisplayName As String * cMaxFile
    szTypeName As String * 80
End Type

Private Declare Function SHGetFileInfo Lib "Shell32" _
    Alias "SHGetFileInfoA" ( _
    ByVal pszPath As String, _
    ByVal dwFileAttributes As Long, _
    psfi As TFileInfo, _
    ByVal cbFileInfo As Long, _
    ByVal uFlags As Long) As Long
    
' Constants for FileDateTime
Public Const vbFileCreated = 1
Public Const vbFileModified = 2
Public Const vbFileAccessed = 3

' Private types for dealing with API times
Public Type TFileTime
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type

Public Type TFindData
    dwFileAttributes As Long
    ftCreationTime As TFileTime
    ftLastAccessTime As TFileTime
    ftLastWriteTime As TFileTime
    nFileSizeHigh As Long
    nFileSizeLow As Long
    dwReserved0 As Long
    dwReserved1 As Long
    cFileName As String * cMaxFile
    cAlternateFileName As String * 14
End Type

Public Type TSystemTime
    wYear As Integer
    wMonth As Integer
    wDayOfWeek As Integer
    wDay As Integer
    wHour As Integer
    wMinute As Integer
    wSecond As Integer
    wMilliseconds As Integer
End Type

Public Declare Function GetFileTime Lib "KERNEL32" (ByVal hFile As Long, _
    lpCreationTime As TFileTime, lpLastAccessTime As TFileTime, _
    lpLastWriteTime As TFileTime) As Long

Public Declare Function SetFileTime Lib "KERNEL32" (ByVal hFile As Long, _
    lpCreationTime As TFileTime, lpLastAccessTime As TFileTime, _
    lpLastWriteTime As TFileTime) As Long

Public Declare Function FileTimeToLocalFileTime Lib "KERNEL32" ( _
    lpFileTime As TFileTime, lpLocalFileTime As TFileTime) As Long

Public Declare Function FileTimeToSystemTime Lib "KERNEL32" ( _
    lpFileTime As TFileTime, lpSystemTime As TSystemTime) As Long

Public Declare Function FindFirstFile Lib "KERNEL32" Alias "FindFirstFileA" ( _
    ByVal lpFileName As String, lpFindFileData As TFindData) As Long

Public Declare Function FindNextFile Lib "KERNEL32" Alias "FindNextFileA" ( _
    ByVal hFind As Long, lpFindFileData As TFindData) As Long

Public Declare Function FindClose Lib "KERNEL32" ( _
    ByVal hFindFile As Long) As Long

Declare Function SHFileOperation Lib "Shell32" Alias "SHFileOperationA" ( _
    lpFileOp As TFileOperation) As Long

''@B TFileOperation
Type TFileOperation
    hWnd                    As Long   ' Parent window for dialog
    wFunc                   As Long   ' Copy, move, rename, delete
    pFrom                   As String ' Source
    pTo                     As String ' Destination (delete ignores)
    fFlags                  As Long   ' Options
    fAnyOperationsAborted   As Long   ' Indicates partial failure
    hNameMappings           As Long   ' Completed operations
    lpszProgressTitle       As String ' Title for dialog
End Type
''@E TFileOperation

Const FO_MOVE = &H1
Const FO_COPY = &H2
Const FO_DELETE = &H3
Const FO_RENAME = &H4

Public Const FOF_MULTIDESTFILES = &H1
Public Const FOF_CONFIRMMOUSE = &H2
Public Const FOF_SILENT = &H4                ' Don't create progress/report
Public Const FOF_RENAMEONCOLLISION = &H8
Public Const FOF_NOCONFIRMATION = &H10       ' Don't prompt the user.
Public Const FOF_WANTMAPPINGHANDLE = &H20    ' Fill in SHFILEOPSTRUCT.hNameMappings
                                      ' Must be freed using SHFreeNameMappings
Public Const FOF_ALLOWUNDO = &H40
Public Const FOF_FILESONLY = &H80            ' On *.*, do only files
Public Const FOF_SIMPLEPROGRESS = &H100      ' Means don't show names of files
Public Const FOF_NOCONFIRMMKDIR = &H200      ' Don't confirm making any needed dirs

Const FOF_COPYFLAGS = FOF_MULTIDESTFILES Or FOF_SILENT Or _
                      FOF_RENAMEONCOLLISION Or FOF_NOCONFIRMATION Or _
                      FOF_ALLOWUNDO Or FOF_FILESONLY Or _
                      FOF_SIMPLEPROGRESS Or FOF_NOCONFIRMMKDIR
Const FOF_MOVEFLAGS = FOF_COPYFLAGS
Const FOF_RENAMEFLAGS = FOF_COPYFLAGS
Const FOF_DELETEFLAGS = FOF_SILENT Or FOF_NOCONFIRMATION Or _
                        FOF_ALLOWUNDO Or FOF_SIMPLEPROGRESS

' VB style constants with better names
Public Const vbFileOpMultiDestFiles = &H1
Public Const vbFileOpNoProgress = &H4
Public Const vbFileOpRenameOnCollision = &H8
Public Const vbFileOpNoConfirm = &H10
Public Const vbFileOpRecycleBin = &H40
Public Const vbFileOpFilesOnly = &H80
Public Const vbFileOpSimpleProgress = &H100
Public Const vbFileOpNoConfirmMkDir = &H200

Private dxyShell As Long

Public Const vbIconRealSize = -1
Public Const vbIconDefault = -2
Public Const vbIconSmall = -3
Public Const vbIconShell = -4

Function FileDateTime(sPath As String, _
                       Optional vCreation As Variant, _
                       Optional vAccess As Variant) As Variant
    If IsMissing(vCreation) And IsMissing(vAccess) Then
        FileDateTime = VBA.FileDateTime(sPath)
        Exit Function
    End If
            
    Dim fnd As TFindData
    Dim ftCreate As TFileTime, ftAccess As TFileTime, ftModify As TFileTime
    Dim hFind As Long, f As Boolean, stime As TSystemTime
    hFind = FindFirstFile(sPath, fnd)
    If hFind = INVALID_HANDLE_VALUE Then GoTo FailFileDateTime
    FindClose hFind
    vCreation = SysToVBTime(fnd.ftCreationTime)
    vAccess = SysToVBTime(fnd.ftLastAccessTime)
    FileDateTime = SysToVBTime(fnd.ftLastWriteTime)
    Exit Function
    
FailFileDateTime:
    Select Case Err.LastDllError
    ' Generate common errors
    Case ERROR_FILE_NOT_FOUND
        Error (53)  ' File not found
    Case ERROR_PATH_NOT_FOUND
        Error (76)  ' Path not found
    Case ERROR_ACCESS_DENIED, ERROR_INVALID_HANDLE
        Error (75)  ' Path/file access error
    Case Else
        Error (5)   ' Invalid procedure call
    End Select
End Function

Function SysToVBTime(ft As TFileTime) As Variant
    Dim f As Boolean, ftl As TFileTime, st As TSystemTime
    If ft.dwLowDateTime = 0 And ft.dwHighDateTime = 0 Then Exit Function
    f = FileTimeToLocalFileTime(ft, ftl)
    If f Then f = FileTimeToSystemTime(ftl, st)
    If f = False Then Exit Function
    SysToVBTime = DateSerial(st.wYear, st.wMonth, st.wDay) + _
                  TimeSerial(st.wHour, st.wMinute, st.wSecond)
End Function

''@B ReplaceFile
Function ReplaceFile(sOld As String, sTmp As String) As Boolean
    Dim fnd As TFindData, hFind As Long, hOld As Long, f As Boolean
    ' Get file time and attributes of old file
    If FindFirstFile(sOld, fnd) = -1 Then GoTo ReplaceFail
    ' Replace by deleting old and renaming new to old
    On Error GoTo ReplaceFail
    VBA.Kill sOld
    Name sTmp As sOld
    On Error GoTo 0
    ' Assign old attributes and time to new file
    hOld = lopen(sOld, OF_WRITE Or OF_SHARE_DENY_WRITE)
    f = SetFileTime(hOld, fnd.ftCreationTime, _
                    fnd.ftLastAccessTime, fnd.ftLastWriteTime)
    lclose hOld
    If f = False Then GoTo ReplaceFail
    f = SetFileAttributes(sOld, fnd.dwFileAttributes)
    If f Then ReplaceFile = True
ReplaceFail:
End Function
''@E ReplaceFile

' Better version of FileCopy and matching FileMove,
' FileDelete, and FileRename

''@B FileCopy
Function FileCopy(sSrc As String, sDst As String, _
                  Optional vOptions As Variant, _
                  Optional vTitle As Variant, _
                  Optional vOwner As Variant) As Boolean
    If HasShell Then
        Dim fo As TFileOperation, f As Long
        fo.wFunc = FO_COPY
        fo.pFrom = sSrc
        fo.pTo = sDst
        If Not IsMissing(vOptions) Then fo.fFlags = vOptions
        If Not IsMissing(vTitle) Then fo.lpszProgressTitle = vTitle
        If Not IsMissing(vOwner) Then fo.hWnd = vOwner
        ' Mask out invalid flags
        fo.fFlags = fo.fFlags And FOF_COPYFLAGS
        f = SHFileOperation(fo)
        FileCopy = (f = 0)
    Else
        ' For Windows NT 3.51
        On Error Resume Next
        ' FileCopy expects full name of destination file
        VBA.FileCopy sSrc, sDst
        If Err Then
            Err = 0
            ' Better FileCopy can handle destination directory
            sDst = NormalizePath(sDst) & GetFileBaseExt(sSrc)
            VBA.FileCopy sSrc, sDst
        End If
        ' Enhance further to emulate SHFileOperation options
        ' such as validation and wild cards
        FileCopy = (Err = 0)
    End If
End Function
''@E FileCopy

Function FileMove(sSrc As String, sDst As String, _
                  Optional vOptions As Variant, _
                  Optional vTitle As Variant, _
                  Optional vOwner As Variant) As Boolean
    If HasShell Then
        Dim fo As TFileOperation, f As Long
        fo.wFunc = FO_MOVE
        fo.pFrom = sSrc
        fo.pTo = sDst
        If Not IsMissing(vOptions) Then fo.fFlags = vOptions
        If Not IsMissing(vTitle) Then fo.lpszProgressTitle = vTitle
        If Not IsMissing(vOwner) Then fo.hWnd = vOwner
        ' Mask out invalid flags
        fo.fFlags = fo.fFlags And FOF_COPYFLAGS
        f = SHFileOperation(fo)
        FileMove = (f = 0)
    Else
        ' Windows NT 3.51
        On Error Resume Next
        ' Name actually moves
        Name sSrc As sDst
        If Err Then ' Probably you gave directory destination
            Err = 0
            sDst = NormalizePath(sDst) & GetFileBaseExt(sSrc)
            Name sSrc As sDst
        End If
        ' Enhance further to emulate SHFileOperation options
        ' such as validation and wild cards
        FileMove = (Err = 0)
    End If
End Function

Function FileRename(sSrc As String, sDst As String, _
                    Optional vOptions As Variant, _
                    Optional vTitle As Variant, _
                    Optional vOwner As Variant) As Boolean
    If HasShell Then
        Dim fo As TFileOperation, f As Long
        If Not IsMissing(vOwner) Then fo.hWnd = vOwner
        fo.wFunc = FO_RENAME
        fo.pFrom = sSrc
        fo.pTo = sDst
        If Not IsMissing(vOptions) Then fo.fFlags = vOptions
        If Not IsMissing(vTitle) Then fo.lpszProgressTitle = vTitle
        If Not IsMissing(vOwner) Then fo.hWnd = vOwner
        ' Mask out invalid flags
        fo.fFlags = fo.fFlags And FOF_RENAMEFLAGS
        f = SHFileOperation(fo)
        FileRename = (f = 0)
    Else
        ' Windows NT 3.51
        On Error Resume Next
        Name sSrc As sDst
        FileRename = (Err = 0)
        ' Enhance further to emulate SHFileOperation options
        ' such as validation and wild cards
    End If
End Function

Function FileDelete(sSrc As String, _
                    Optional vOptions As Variant, _
                    Optional vTitle As Variant, _
                    Optional vOwner As Variant) As Boolean
    If HasShell Then
        Dim fo As TFileOperation, f As Long
        If Not IsMissing(vOwner) Then fo.hWnd = vOwner
        fo.wFunc = FO_DELETE
        fo.pFrom = sSrc
        ' fo.pTo = sNullStr
        If Not IsMissing(vOptions) Then fo.fFlags = vOptions
        If Not IsMissing(vTitle) Then fo.lpszProgressTitle = vTitle
        If Not IsMissing(vOwner) Then fo.hWnd = vOwner
        ' Mask out invalid flags
        fo.fFlags = fo.fFlags And FOF_DELETEFLAGS
        f = SHFileOperation(fo)
        FileDelete = (f = 0)
    Else
        ' Windows NT 3.51
        On Error Resume Next
        Kill sSrc
        FileDelete = (Err = 0)
        ' Enhance further to emulate SHFileOperation options
        ' such as validation and wild cards
    End If
End Function

''@B LoadPicture
Function LoadPicture(Optional vPicture As Variant, _
                     Optional vIconType As Variant) As Picture
    Dim hIcon As Long, sExt As String, xy As Long, af As Long
    ' If no picture, return Nothing (clears picture)
    If IsMissing(vPicture) Then Exit Function
    ' Use default LoadPicture for all except icons with argument
    sExt = GetFileExt(CStr(vPicture))
    If UCase$(sExt) <> ".ICO" Or IsMissing(vIconType) Then
        Set LoadPicture = VB.LoadPicture(vPicture)
        Exit Function
    End If
    
    Select Case vIconType
    Case vbIconRealSize
        xy = 0: af = LR_LOADFROMFILE Or LR_LOADREALSIZE
    Case vbIconSmall
        xy = 16: af = LR_LOADFROMFILE
    Case vbIconShell
        Static dxyShell As Long
        ' Get icon size from system ImageList (FILEINFO.BAS)
        If dxyShell = 0 Then dxyShell = GetShellIconSize()
        xy = dxyShell: af = LR_LOADFROMFILE
    Case Is > 0
        xy = vIconType: af = LR_LOADFROMFILE
    Case Else ' Includes vbIconDefault
        xy = 0: af = LR_LOADFROMFILE Or LR_DEFAULTSIZE
    End Select
    hIcon = LoadImage(0&, CStr(vPicture), IMAGE_ICON, xy, xy, af)
    ' If this fails, use original load
    If hIcon <> hNull Then
        Set LoadPicture = IconToPicture(hIcon)
    Else
        Set LoadPicture = VB.LoadPicture(vPicture)
    End If
End Function
''@E LoadPicture

''@B LoadResPicture
Function LoadResPicture(vRes As Variant, iResType As Integer, _
                        Optional vIconType As Variant) As Picture
    Dim hIcon As Long, sExt As String, xy As Long, af As Long
    ' Can't use LoadImage in environment--have to make do with default
    If Not IsExe() Then
        If IsMissing(vIconType) Or (iResType <> vbResIcon) Then
            Set LoadResPicture = VB.LoadResPicture(vRes, iResType)
            Exit Function
        End If
    End If
    
    Select Case vIconType
    Case vbIconRealSize
        xy = 0: af = LR_LOADFROMFILE Or LR_LOADREALSIZE
    Case vbIconSmall
        xy = 16: af = LR_LOADFROMFILE
    Case vbIconShell
        If dxyShell = 0 Then dxyShell = GetShellIconSize()
        xy = dxyShell: af = LR_LOADFROMFILE
    Case Is > 0
        xy = vIconType: af = LR_LOADFROMFILE
    Case Else ' Includes vbIconDefault
        xy = 0: af = LR_LOADFROMFILE Or LR_DEFAULTSIZE
    End Select
    If TypeName(vRes) = "String" Then
        hIcon = LoadImage(App.hInstance, CStr(vRes), IMAGE_ICON, xy, xy, af)
    Else
        hIcon = LoadImage(App.hInstance, CLng(vRes), IMAGE_ICON, xy, xy, af)
    End If
    If hIcon <> hNull Then
        Set LoadResPicture = IconToPicture(hIcon)
    Else
        Set LoadResPicture = VB.LoadResPicture(vRes, iResType)
    End If
End Function
''@E LoadResPicture

Function GetShellIconSize() As Long
    Dim hImlst As Long, fi As TFileInfo, cx As Long, cy As Long
    hImlst = SHGetFileInfo(".", 0, fi, Len(fi), _
                           SHGFI_SYSICONINDEX Or SHGFI_SHELLICONSIZE)
    If ImageList_GetIconSize(hImlst, cx, cy) Then
        GetShellIconSize = cx
    Else
        GetShellIconSize = -1
    End If
End Function

#End If
