Attribute VB_Name = "MBytes"
Option Explicit
#If Win32 Then
DefLng A-Z
#Else
DefInt A-Z
#End If

''@B StrToBytes
Sub StrToBytes(ab() As Byte, s As String)
    If IsArrayEmpty(ab) Then
        ' Just assign to empty array
#If Win32 Then
        ab = StrConv(s, vbFromUnicode)
#Else
        ab = s
#End If
    Else
        Dim cab ' As SysInt
        ' Copy to existing array, padding or truncating if needed
        cab = UBound(ab) - LBound(ab) + 1
        If Len(s) < cab Then s = s & String(cab - Len(s), 0)
        CopyMemory ab(LBound(ab)), ByVal s, cab
    End If
End Sub
''@E StrToBytes

''@B BytesToStr
Function BytesToStr(ab() As Byte) As String
#If Win32 Then
    BytesToStr = StrConv(ab(), vbUnicode)
#Else
    BytesToStr = ab()
#End If
End Function
''@E BytesToStr

''@B BytesToWord
Function BytesToWord(abBuf() As Byte, iOffset As Long) As Integer
    BugAssert iOffset <= UBound(abBuf) - 1
    Dim w As Integer
    CopyMemory w, abBuf(iOffset), 2
    BytesToWord = w
End Function
''@E BytesToWord

Function BytesToDWord(abBuf() As Byte, iOffset As Long) As Long
    BugAssert iOffset <= UBound(abBuf) - 4
    Dim dw As Long
    CopyMemory dw, abBuf(iOffset), 4
    BytesToDWord = dw
End Function

''@B BytesToWord
Sub BytesFromWord(w As Integer, abBuf() As Byte, iOffset As Long)
    BugAssert iOffset <= UBound(abBuf) - 1
    CopyMemory abBuf(iOffset), w, 2
End Sub
''@E BytesToWord

' Read string with length in first byte
''@B BytesToPStr
Function BytesToPStr(ab() As Byte, iOffset As Long) As String
    BugAssert iOffset <= UBound(ab)
    BytesToPStr = MidBytes(ab, iOffset + 1, ab(iOffset))
End Function
''@E BytesToPStr

Sub BytesFromDWord(dw As Long, abBuf() As Byte, iOffset As Long)
    BugAssert iOffset <= UBound(abBuf) - 4
    CopyMemory abBuf(iOffset), dw, 4
End Sub

'' Emulate relevant Basic string functions for arrays of bytes:
''     Len$             LenBytes
''     Mid$ function    MidBytes
''     Mid$ statement   InsBytes sub
''     Left$            LeftBytes
''     Right$           RightBytes

' LenBytes - Emulates Len for array of bytes
Function LenBytes(ab() As Byte) As Long
    LenBytes = UBound(ab) - LBound(ab) + 1
End Function

' MidBytes - emulates Mid$ function for array of bytes
' (Note that MidBytes does not emulate Mid$ exactly--string fields
' in byte arrays are often null-padded, and MidBytes can extract
' non-null portion.)
''@B MidBytes
Function MidBytes(ab() As Byte, iOffset, Optional vLen As Variant, _
                  Optional vToNull As Variant) As String
    BugAssert iOffset < LenBytes(ab) And iOffset >= 0
    Dim s As String, fToNull As Boolean, cab ' As SysInt
    If Not IsMissing(vToNull) Then fToNull = vToNull
    ' Calculate length
    If IsMissing(vLen) Then
        cab = LenBytes(ab) - iOffset
    Else
        cab = vLen
    End If
    ' Assign and return string
    s = String$(cab, 0)
    CopyMemory ByVal s, ab(iOffset), cab
    If fToNull Then
        cab = InStr(s, vbNullChar)
        MidBytes = Left$(s, cab - 1)
    Else
        MidBytes = s
    End If
End Function
''@E MidBytes

' InsBytes - Emulates Mid$ statement for array of bytes
' (Note that InsBytes does not emulate Mid$ exactly--it inserts
' a null-padded string into a fixed-size field in order to work
' better with common use of byte arrays.)
Sub InsBytes(sIns As String, ab() As Byte, iOffset, _
             Optional vLen As Variant)
    BugAssert iOffset < LenBytes(ab) And iOffset >= 0
    Dim cab ' As SysInt
    ' Calculate length
    If IsMissing(vLen) Then
        cab = Len(sIns)
    Else
        cab = vLen
        ' Null-pad insertion string if too short
        If (Len(sIns) < cab) Then
            sIns = sIns & String$(cab - Len(sIns), 0)
        End If
    End If
    BugAssert Len(sIns) <= (LenBytes(ab) - iOffset)
    ' Insert string
    CopyMemory ab(iOffset), ByVal sIns, cab
End Sub

' LeftBytes - Emulates Left$ function for array of bytes
Function LeftBytes(ab() As Byte, iLen) As String
    Dim s As String
    s = String$(iLen, 0)
    CopyMemory ByVal s, ab(LBound(ab)), iLen
    LeftBytes = s
End Function

' RightBytes - Emulates Right$ function for array of bytes
Function RightBytes(ab() As Byte, iLen) As String
    Dim s As String
    s = String$(iLen, 0)
    CopyMemory ByVal s, ab(UBound(ab) - iLen + 1), iLen
    RightBytes = s
End Function

' FillBytes - Fills field in array of bytes with given byte
Sub FillBytes(ab() As Byte, b As Byte, iOffset, iLen)
    BugAssert (iOffset < LenBytes(ab)) And (iOffset >= 0)
    BugAssert (iLen < (LenBytes(ab) - iOffset)) And (iOffset >= 0)
    Dim i ' As SysInt
    For i = iOffset To iOffset + iLen - 1
        ab(i) = b
    Next
End Sub

' InStrBytes is not implemented because a simple version would
' simply be equivalent to InStr(ab(), s). This creates a temporary
' string for ab() on every call. An efficient version that works
' directly on arrays of bytes could be written in C.


