VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CEditor"
Attribute VB_Creatable = True
Attribute VB_Exposed = False
Option Explicit
#If Win32 Then
DefLng A-Z
#Else
DefInt A-Z
#End If

'$ Uses UTILITY.BAS COMDLG.BAS

''@B rtb
' Internal TextBox
#If fRichText Then
Private rtb As RichTextBox
#Else
Private rtb As TextBox
#End If
''@E rtb

Const ordAll = 0
Const ordDown = 1
Const ordUp = 2

Public SearchOptionCase As Boolean
Public SearchOptionWord As Boolean  ' Left as an exercise

Public FindWhatList As New Collection
Public ReplaceWithList As New Collection

Private ordDir As Integer
Private errLast As Variant

Private cCharPerTab As Integer
Private sFileCur As String
Private sFilter As String, iFilter As Integer
Private sDirLast As String
Private f16FlagsFile As Integer
Private f16FlagsFont As Integer
Private f16FlagsPrinter As Integer
Private ordLoadMode As Integer

''@B OverWrite
Public OverWrite As Boolean
''@E OverWrite

''@B Class_Initialize
Sub Class_Initialize()
    cCharPerTab = 8
    sDirLast = CurDir$
    ordLoadMode = rtfText
    If sFilter = sEmpty Then
#If fRichText Then
        sFilter = "Text Files (*.txt)|*.txt|Rich Text Files (*.rtf)|*.rtf|" & _
                  "All Files (*.*)|*.*"
#Else
        sFilter = "Text Files (*.txt)|*.txt|All Files (*.*)|*.*"
#End If
    End If
End Sub
''@E Class_Initialize

''@B Create
#If fRichText Then
Function Create(rtbA As RichTextBox) As Boolean
#Else
Function Create(rtbA As TextBox) As Boolean
#End If
    Create = True
    On Error GoTo CreateError
    ' Set properties for editing
    If Not rtbA.MultiLine Then GoTo CreateError
    If rtbA.ScrollBars <> vbBoth Then GoTo CreateError
#If fRichText = 0 Then
    rtbA.PasswordChar = sEmpty
#End If
    rtbA.MaxLength = 0
    Set rtb = rtbA
    DirtyBit = False
    Exit Function
CreateError:
    Create = False
End Function
''@E Create

Property Get LastError() As Long
    LastError = errLast
End Property

Sub FileNew()
    rtb.Text = sEmpty
    FileName = sEmpty
    DirtyBit = False
End Sub

''@B FileOpen
Function FileOpen() As Boolean
    FileOpen = False
    Dim f As Boolean, sFile As String, sFileTitle As String
    f = VBGetOpenFileName( _
        vFileName:=sFile, _
        vFileTitle:=sFileTitle, _
        vInitDir:=sDirLast, _
        vFlags:=cdlOFNFileMustExist Or cdlOFNHideReadOnly, _
        vFilter:=sFilter)
    If f And sFile <> sEmpty Then
        On Error Resume Next
        sDirLast = CurDir$
        LoadFile sFile, ordLoadMode
        If Err Then
            MsgBox "Can't open file: " & Error(Err)
        Else
            FileName = sFile
        End If
        FileOpen = True
    End If
End Function
''@E FileOpen

Property Get FileOpenFilter() As String
    FileOpenFilter = sFilter
End Property

Property Let FileOpenFilter(sFilterA As String)
    sFilter = sFilter & sFilterA
End Property

Property Get LoadMode() As Integer
#If fRichText Then
    FileOpenFilter = sFilter
#Else
    FileOpenFilter = 1 ' Always text
#End If
End Property

Property Let LoadMode(ordLoadModeA As Integer)
#If fRichText Then
    ordLoadMode = ordLoadModeA
#End If
End Property

Sub FileSave()
    If FileName = sEmpty Then
        FileSaveAs
    Else
        SaveFile FileName, ordLoadMode
    End If
    DirtyBit = False
End Sub

''@B FileSaveAs
Function FileSaveAs() As Boolean
    FileSaveAs = False
    Dim f As Boolean, sFile As String, sFileTitle As String
    f = VBGetSaveFileName( _
        vFileName:=sFile, _
        vFileTitle:=sFileTitle, _
        vInitDir:=sDirLast, _
        vFlags:=cdlOFNFileMustExist Or cdlOFNHideReadOnly, _
        vFilter:=sFilter)
    If f And sFile <> sEmpty Then
        On Error Resume Next
        FileName = sFile
        SaveFile FileName, ordLoadMode
        DirtyBit = True
    End If
End Function
''@E FileSaveAs

Sub FilePrint()
    MsgBox "Left as an exercise for the reader"
End Sub

Sub FilePageSetup()
    MsgBox "Left as an exercise for the reader"
End Sub

Sub OptionColor(fBackColor As Integer)
    Dim clr As Long, f As Boolean
    If fBackColor Then
        clr = vbWhite
    Else
        clr = vbBlack
    End If
    f = VBChooseColor(vColor:=clr, _
                      vFlags:=cdlCCPreventFullOpen Or cdlCCRGBInit)
    If Not f Then Exit Sub
    If fBackColor Then
        BackColor = clr
    Else
        ForeColor = clr
    End If
End Sub

Sub OptionFont()
    Dim f As Boolean, fnt As Font, clr As Long
    Set fnt = rtb.Font
    clr = ForeColor
    f = VBChooseFont(vFont:=fnt, vColor:=clr, _
                     vFlags:=cdlCFEffects Or cdlCFBoth)
    If Not f Then Exit Sub
    Set rtb.Font = fnt
    ForeColor = clr
    rtb.Refresh
End Sub

' This function returns 0/1 rather than True/False for
' easier use with check boxes.
Private Function GetBit(iValue As Integer, iBitPos As Integer) As Integer
    If iValue And (2 ^ iBitPos) Then
        GetBit = 1
    Else
        GetBit = 0
    End If
End Function

Property Get FileTitle() As String
    FileTitle = GetFileBaseExt(FileName)
End Property

''@B DirtyDialog
Public Function DirtyDialog() As Integer
    Dim s As String
    DirtyDialog = True ' Assume success
    ' Done if no dirty file to save
    If Not DirtyBit Then Exit Function
    ' Prompt for action if dirty file
    s = "File not saved: " & FileName & ". Save now?"
    Select Case MsgBox(s, vbYesNoCancel)
    Case vbYes
        ' Save old file
        FileSave
    Case vbCancel
        ' User wants to terminate file change
        DirtyDialog = False
    Case vbNo
        ' Do nothing if user wants to throw away changes
    End Select
End Function
''@E DirtyDialog

' Cut, copy, paste methods

Sub EditCopy()
    Clipboard.SetText rtb.SelText
End Sub

Sub EditCut()
    Clipboard.SetText rtb.SelText
    rtb.SelText = ""
End Sub

Sub EditDelete()
    If rtb.SelLength = 0 Then rtb.SelLength = 1
    rtb.SelText = ""
End Sub

Sub EditPaste()
    rtb.SelText = Clipboard.GetText()
End Sub

''@B EditSelectAll
Sub EditSelectAll()
    rtb.SelStart = 0
    rtb.SelLength = 65000
End Sub
''@E EditSelectAll

' SendMessage enhancement methods and properties

''@B Undo
Sub EditUndo()
    Call SendMessageAsLong(rtb.hWnd, EM_UNDO, 0, 0&)
End Sub

Sub ClearUndo()
    Call SendMessageAsLong(rtb.hWnd, EM_EMPTYUNDOBUFFER, 0, 0&)
End Sub

Property Get CanUndo() As Boolean
    CanUndo = SendMessageAsLong(rtb.hWnd, EM_CANUNDO, 0, 0&)
End Property
''@E Undo

''@B GetEditStatus
Sub GetEditStatus(iLine As Long, cLine As Long, _
                  iCol As Long, cCol As Long, _
                  iChar As Long, cChar As Long)
    Dim i As Long
    ' Count of lines
    cLine = SendMessageAsLong(rtb.hWnd, EM_GETLINECOUNT, 0, 0&)
    ' Current line (zero adjusted)
    iLine = 1 + SendMessageAsLong(rtb.hWnd, EM_LINEFROMCHAR, _
                                  Lo(rtb.SelStart), 0&)
    ' Current character
    iChar = rtb.SelStart + 1
    ' Length is position of last line plus length of last line
    cChar = SendMessageAsLong(rtb.hWnd, EM_LINEINDEX, _
                              Lo(cLine - 1), 0&)
    i = SendMessageAsLong(rtb.hWnd, EM_LINELENGTH, Lo(cChar - 1), 0&)
    cChar = cChar + i
    ' Column count is current line length
    cCol = SendMessageAsLong(rtb.hWnd, EM_LINELENGTH, _
                             Lo(iChar - 1), 0&)
    ' Column is current position minus position of line start
    i = SendMessageAsLong(rtb.hWnd, EM_LINEINDEX, Lo(iLine - 1), 0&)
    iCol = iChar - i
End Sub
''@E GetEditStatus

Property Get Line() As Long
    ' Current line (zero adjusted)
    Line = 1 + SendMessageAsLong(rtb.hWnd, EM_LINEFROMCHAR, _
                                 Lo(rtb.SelStart), 0&)
End Property

Property Let Line(iLineA As Long)
    On Error Resume Next
    rtb.SelStart = LinePosition(iLineA - 1) - 1
End Property

Property Get Lines() As Long
    ' Count of lines
    Lines = SendMessageAsLong(rtb.hWnd, EM_GETLINECOUNT, 0, 0&)
End Property

Property Get Character() As Long
    ' Current character
    Character = rtb.SelStart + 1
End Property

Property Let Character(iPos As Long)
    rtb.SelStart = iPos - 1
End Property

Property Get Characters() As Long
    Dim i As Long
    ' Length is position of last line plus length of last line
    i = SendMessageAsLong(rtb.hWnd, EM_LINEINDEX, Lo(Lines - 1), 0&)
    Characters = i + SendMessageAsLong(rtb.hWnd, EM_LINELENGTH, _
                                       Lo(Lines - 1), 0&)
End Property

Property Get Column() As Long
    ' Column is current position minus position of line start
    Dim i As Long
    i = SendMessageAsLong(rtb.hWnd, EM_LINEINDEX, Lo(Line - 1), 0&)
    Column = Character - i
End Property

Property Let Column(iColA As Long)
    rtb.SelStart = LinePosition + iColA - 2
End Property

Property Get Columns() As Long
    ' Column count is current line length
    Columns = SendMessageAsLong(rtb.hWnd, EM_LINELENGTH, _
                                Lo(Character - 1), 0&)
End Property

Property Get Percent() As Integer
    Percent = (Character / (Characters + 1)) * 100
End Property

Property Let Percent(iA As Integer)
    rtb.SelStart = Characters * (iA / 100)
End Property

Public Function LinePosition(Optional iLine As Variant) As Long
    If IsMissing(iLine) Then iLine = -1
    LinePosition = SendMessageAsLong(rtb.hWnd, EM_LINEINDEX, _
                                     Lo(iLine), 0&) + 1
End Function

''@B LineLength
Public Function LineLength(Optional iLine As Variant) As Long
    If IsMissing(iLine) Then iLine = Line
    LineLength = SendMessageAsLong(rtb.hWnd, EM_LINELENGTH, _
                                   Lo(LinePosition(iLine)), 0&)
End Function
''@E LineLength

Public Function LineText(Optional iLine As Variant) As String
    If IsMissing(iLine) Then iLine = Line
    Const cCharMax = 252
    Dim s As String, c As Integer
    s = Space$(cCharMax + 3)
    Mid$(s, 1, 1) = Chr$(cCharMax And &HFF)
    Mid$(s, 2, 1) = Chr$(cCharMax \ 256)
    c = SendMessageAsStr(rtb.hWnd, EM_GETLINE, Lo(iLine - 1), s)
    LineText = Left$(s, c)
End Function

Function GetLineFromChar(iPos As Long) As Long
#If fRichText Then
    GetLineFromChar = rtb.GetLineFromChar(iPos)
#Else
    GetLineFromChar = 1 + SendMessageAsLong(rtb.hWnd, EM_LINEFROMCHAR, _
                                            iPos, 0&)
#End If
End Function

Property Get Tabs() As Integer
    Tabs = cCharPerTab
End Property

Property Let Tabs(cTab As Integer)

    Dim c ' As SysInt
    c = cTab * 4 ' Assume 4 dialog box units per character
    c = SendMessageForInt(rtb.hWnd, EM_SETTABSTOPS, 1, c)

    cCharPerTab = cTab

End Property

Sub Scroll(iVLine As Integer, iHLine As Integer)
    Dim i As Long
    i = iHLine
    i = i * 65535
    i = (i And &HFFFF&) + iVLine
    i = SendMessageAsLong(rtb.hWnd, EM_LINESCROLL, 0, i)
End Sub

' Find and Replace option properties

''@B GetFindWhat
Property Get FindWhat() As String
    If FindWhatList.Count Then
        FindWhat = FindWhatList(FindWhatList.Count)
    Else
        FindWhat = sEmpty
    End If
End Property
''@E GetFindWhat

''@B LetFindWhat
Property Let FindWhat(sWhatA As String)
    Dim v As Variant, i As Integer ' i = 0
    For i = 1 To FindWhatList.Count
        ' If item is in list, move to end of list
        If FindWhatList(i) = sWhatA Then
            FindWhatList.Remove i
            FindWhatList.Add sWhatA
            Exit Property
        End If
    Next
    ' If item isn't in list, add it
    FindWhatList.Add sWhatA
End Property
''@E LetFindWhat

Property Get ReplaceWith() As String
    If ReplaceWithList.Count Then
        ReplaceWith = ReplaceWithList(ReplaceWithList.Count)
    Else
        ReplaceWith = sEmpty
    End If
End Property

Property Let ReplaceWith(sWithA As String)
    Dim v As Variant, i As Integer ' i = 0
    For i = 1 To ReplaceWithList.Count
        ' If item is in list, move to end of list
        If ReplaceWithList(i) = sWithA Then
            ReplaceWithList.Remove i
            ReplaceWithList.Add sWithA
            Exit Property
        End If
    Next
    ' If item isn't in list, add it
    ReplaceWithList.Add sWithA
End Property

Property Get SearchOptionDirection() As Integer
    SearchOptionDirection = ordDir
End Property

Property Let SearchOptionDirection(ordDirA As Integer)
    If ordDirA < 0 Or ordDirA > 2 Then ordDirA = 0
    ordDir = ordDirA
End Property

' Search and Replace functions

#If fNoFSearch = 0 Then
''@B SearchFind
Sub SearchFind()
    Dim fnd As New FSearch
    ' Set properties on FSearch form
    fnd.ReplaceMode = False
    Set fnd.Editor = Me
     ' Load, but don't show yet
    Load fnd
    ' Local form destroys automatically
End Sub
''@E SearchFind

Sub SearchFindNext()
    If FindWhat = sEmpty Then
        SearchFind
    Else
        Call FindNext(FindWhat)
    End If
End Sub

Sub SearchReplace()
    Dim replace As New FSearch
    
    ' Set properties on SearchReplace form
    replace.ReplaceMode = True
    Set replace.Editor = Me
    ' Load form (but don't show it yet)
    Load replace
    ' Local form destroys automatically

End Sub
#End If

Function FindNext(Optional sWhat As Variant, Optional fMark As Variant) As Integer
    Dim fWrap As Boolean, fReverse As Boolean

    ' Assume fail
    FindNext = 0
    
    ' Set up options
    If IsMissing(fMark) Then fMark = True
    fWrap = (SearchOptionDirection = ordAll)
    fReverse = (SearchOptionDirection = ordUp)
    If IsMissing(sWhat) Then sWhat = FindWhat
    
    ' Search for string
    Dim s As String, iPos As Integer
    s = rtb.Text
    iPos = rtb.SelStart
    If fReverse Then
        iPos = InStrR(iPos, s, sWhat, -SearchOptionCase)
    Else
        iPos = InStr(iPos + 2, s, sWhat, -SearchOptionCase)
    End If

    ' If not found, wrap if appropriate
    If iPos = 0 Then
        If fWrap Then
            If fReverse Then
                iPos = InStrR(Len(s), s, sWhat, -SearchOptionCase)
            Else
                iPos = InStr(1, s, sWhat, -SearchOptionCase)
            End If
        End If
    End If
    
    ' Mark found text if requested
    If fMark And iPos Then
        rtb.SelStart = iPos - 1
        rtb.SelLength = Len(sWhat)
    End If
    FindNext = iPos

End Function

Function ReplaceNext(Optional sWhat As Variant, Optional sWith As Variant) As Integer
        
    ' Assume fail
    ReplaceNext = 0
    
    If IsMissing(sWhat) Then sWhat = FindWhat
    If IsMissing(sWith) Then sWith = ReplaceWith
    If sWhat = sEmpty Then Exit Function
    
    ' If first match not yet found, find it
    Dim i As Integer, s As String
    s = Mid(rtb.Text, rtb.SelStart + 1, Len(sWhat))
    If StrComp(s, sWhat, -SearchOptionCase) Then
        i = FindNext(sWhat)
        If i = 0 Then Exit Function
    Else ' If already found, make sure it's marked
        i = rtb.SelStart + 1
        rtb.SelLength = Len(sWhat)
    End If

    ' Replace text
    If i Then
        rtb.SelText = sWith
        ReplaceNext = i
    End If

End Function

' TextBox properties passed through

Property Get Appearance() As Integer
    Appearance = rtb.Appearance
End Property

''@B BackColor
Property Get BackColor() As Long
    BackColor = rtb.BackColor
End Property

Property Let BackColor(iBackColorA As Long)
    rtb.BackColor = iBackColorA
End Property
''@E BackColor

#If fRichText Then
Property Get BulletIndent() As Integer
    BulletIndent = rtb.BulletIndent
End Property

Property Let BulletIndent(iBulletIndentA As Integer)
    rtb.BulletIndent = iBulletIndentA
End Property
#End If

Property Get Container() As Object
    Set Container = rtb.Container
End Property

Property Set Container(vContainerA As Object)
    Set rtb.Container = vContainerA
End Property

''@B DirtyBit
Property Get DirtyBit() As Boolean
    DirtyBit = SendMessageAsLong(rtb.hWnd, EM_GETMODIFY, 0, 0)
End Property

Property Let DirtyBit(fDirtyBitA As Boolean)
    Call SendMessageAsLong(rtb.hWnd, EM_SETMODIFY, fDirtyBitA, 0)
End Property
''@E DirtyBit

Property Get DataChanged() As Boolean
    DataChanged = rtb.DataChanged
End Property

Property Let DataChanged(fDataChangedA As Boolean)
    rtb.DataChanged = fDataChangedA
End Property

Property Get DataField() As String
    DataField = rtb.DataField
End Property

Property Let DataField(sDataFieldA As String)
    rtb.DataField = sDataFieldA
End Property

#If fRichText Then
Property Get DisableNoScroll() As Boolean
    DisableNoScroll = rtb.DisableNoScroll
End Property

Property Let DisableNoScroll(iDisableNoScrollA As Boolean)
    rtb.DisableNoScroll = iDisableNoScrollA
End Property
#End If

Property Get DragIcon() As Picture
    Set DragIcon = rtb.DragIcon
End Property

Property Set DragIcon(vDragIconA As Picture)
    rtb.DragIcon = vDragIconA
End Property

Property Get DragMode() As Integer
    DragMode = rtb.DragMode
End Property

Property Let DragMode(iDragModeA As Integer)
    rtb.DragMode = iDragModeA
End Property

Property Get Enabled() As Boolean
    Enabled = rtb.Enabled
End Property

Property Let Enabled(iEnabledA As Boolean)
    rtb.Enabled = iEnabledA
End Property

Property Get FileName() As String
    FileName = sFileCur
End Property

Property Let FileName(sFileCurA As String)
    If sFileCurA = sEmpty Then sFileCur = sFileCurA: Exit Property
    ' Late discovered incompatibility means you can't load a text file
    ' with CR/LF in them because these are considered "binary" characters.
    ' So text load files the old way.
#If fRichTextBox Then
    If vFileType = rtfRTF Then
        ' Pass through RichTextBox method
        rtb.FileName = sFileCurA
        sFileCur = sFileCurA
        DirtyBit = False
        Exit Sub
    End If
#End If
    Dim s As String
    s = GetFileText(sFileCurA, errLast)
    If s = sEmpty Then
        errLast = 53 ' File not found
    ElseIf Len(s) <> FileLen(sFileCurA) Then
        ' Use this to mean file too long
        errLast = 62 ' Input past end of file
    Else
        rtb.Text = s
        sFileCur = sFileCurA
        DirtyBit = False
    End If
End Property

Property Get Font() As Font
    Set Font = rtb.Font
End Property

Property Set Font(vFontA As Font)
    rtb.Font = vFontA
End Property

Property Get ForeColor() As Long
#If fRichText Then
    ForeColor = rtb.SelColor
#Else
    ForeColor = rtb.ForeColor
#End If
End Property

Property Let ForeColor(clrForeColorA As Long)
With rtb
#If fRichText Then
    Dim fEnabled As Boolean, iStart, iLength ' As SysInt
    ' Save selection
    fEnabled = .Enabled
    .Enabled = False
    iStart = .SelStart
    iLength = .SelLength
    .SelColor = clrForeColorA
    ' Select all and change color
    .SelStart = 0
    .SelLength = Characters
    .SelColor = clrForeColorA
    ' Restore selection
    .SelStart = iStart
    .SelLength = iLength
    .Enabled = fEnabled
#Else
    .ForeColor = clrForeColorA
#End If
End With
End Property

Property Get Height() As Single
    Height = rtb.Height
End Property

Property Let Height(rHeightA As Single)
    rtb.Height = rHeightA
End Property

Property Get HelpContextId() As Long
    HelpContextId = rtb.HelpContextId
End Property

Property Let HelpContextId(iHelpContextIdA As Long)
    rtb.HelpContextId = iHelpContextIdA
End Property

Property Get HideSelection() As Boolean
    HideSelection = rtb.HideSelection
End Property

Property Get hWnd() ' As SysInt
    hWnd = rtb.hWnd
End Property

Property Get Left() As Single
    Left = rtb.Left
End Property

Property Let Left(rLeftA As Single)
    rtb.Left = rLeftA
End Property

#If fRichText = 0 Then
Property Get LinkItem() As Integer
    LinkItem = rtb.LinkItem
End Property

Property Let LinkItem(iLinkItemA As Integer)
    rtb.LinkItem = iLinkItemA
End Property

Property Get LinkMode() As Integer
    LinkMode = rtb.LinkMode
End Property

Property Let LinkMode(iLinkModeA As Integer)
    rtb.LinkMode = iLinkModeA
End Property

Property Get LinkTimeout() As Long
    LinkTimeout = rtb.LinkTimeout
End Property

Property Let LinkTimeout(iLinkTimeoutA As Long)
    rtb.LinkTimeout = iLinkTimeoutA
End Property

Property Get LinkTopic() As String
    LinkTopic = rtb.LinkTopic
End Property

Property Let LinkTopic(sLinkTopicA As String)
    rtb.LinkTopic = sLinkTopicA
End Property
#End If

Property Get Locked() As Boolean
    Locked = rtb.Locked
End Property

Property Let Locked(fLockedA As Boolean)
    rtb.Locked = fLockedA
End Property

Property Get MouseIcon() As Picture
    Set MouseIcon = rtb.MouseIcon
End Property

Property Set MouseIcon(vMouseIconA As Picture)
    rtb.MouseIcon = vMouseIconA
End Property

Property Get MousePointer() As Integer
    MousePointer = rtb.MousePointer
End Property

Property Let MousePointer(iMousePointerA As Integer)
    rtb.MousePointer = iMousePointerA
End Property

''@B Parent
Property Get Parent() As Object
    Set Parent = rtb.Parent
End Property

Property Set Parent(vParentA As Object)
    Set rtb.Parent = vParentA
End Property
''@E Parent

#If fRichText Then
Property Get SelAlignment() As Long
    SelAlignment = rtb.SelAlignment
End Property

Property Let SelAlignment(iSelAlignmentA As Long)
    rtb.SelAlignment = iSelAlignmentA
End Property

Property Get SelBold() As Boolean
    SelBold = rtb.SelBold
End Property

Property Let SelBold(fSelBoldA As Boolean)
    rtb.SelBold = fSelBoldA
End Property

Property Get SelBullet() As Boolean
    SelBullet = rtb.SelBullet
End Property

Property Let SelBullet(fSelBulletA As Boolean)
    rtb.SelBullet = fSelBulletA
End Property

Property Get SelCharOffset() As Integer
    SelCharOffset = rtb.SelCharOffset
End Property

Property Let SelCharOffset(iSelCharOffsetA As Integer)
    rtb.SelCharOffset = iSelCharOffsetA
End Property
#End If

Property Get SelColor() As Long
#If fRichText Then
    SelColor = rtb.SelColor
#Else
    SelColor = rtb.ForeColor
#End If
End Property

Property Let SelColor(clrSelColorA As Long)
#If fRichText Then
    rtb.SelColor = clrSelColorA
#Else
    rtb.ForeColor = clrSelColorA
#End If
End Property

Property Get SelFontName() As String
#If fRichText Then
    SelFontName = rtb.SelFontName
#Else
    SelFontSize = rtb.Font.Name
#End If
End Property

Property Let SelFontName(sSelFontNameA As String)
#If fRichText Then
    rtb.SelFontName = sSelFontNameA
#Else
    rtb.Font.Size = sSelFontNameA
#End If
End Property

Property Get SelFontSize() As Long
#If fRichText Then
    SelFontSize = rtb.SelFontSize
#Else
    SelFontSize = rtb.Font.Size
#End If
End Property

Property Let SelFontSize(clrSelFontSizeA As Long)
#If fRichText Then
    rtb.SelFontSize = clrSelFontSizeA
#Else
    rtb.Font.Size = clrSelFontSizeA
#End If
End Property

#If fRichText Then
Property Get SelHangingIndent() As Integer
    SelHangingIndent = rtb.SelHangingIndent
End Property

Property Let SelHangingIndent(iSelHangingIndentA As Integer)
    rtb.SelHangingIndent = iSelHangingIndentA
End Property

Property Get SelIndent() As Integer
    SelIndent = rtb.SelIndent
End Property

Property Let SelIndent(iSelIndentA As Integer)
    rtb.SelIndent = iSelIndentA
End Property

Property Get SelItalic() As Boolean
    SelItalic = rtb.SelItalic
End Property

Property Let SelItalic(fSelItalicA As Boolean)
    rtb.SelItalic = fSelItalicA
End Property
#End If

Property Get SelLength() As Long
    SelLength = rtb.SelLength
End Property

Property Let SelLength(iSelLengthA As Long)
    rtb.SelLength = iSelLengthA
End Property

#If fRichText Then
Property Get SelRightIndent() As Integer
    SelRightIndent = rtb.SelRightIndent
End Property

Property Let SelRightIndent(iSelRightIndentA As Integer)
    rtb.SelRightIndent = iSelRightIndentA
End Property

Property Get SelRTF() As String
    SelRTF = rtb.SelRTF
End Property

Property Let SelRTF(sSelRTFA As String)
    rtb.SelRTF = sSelRTFA
End Property
#End If

Property Get SelStart() As Long
    SelStart = rtb.SelStart
End Property

Property Let SelStart(iSelStartA As Long)
    rtb.SelStart = iSelStartA
End Property

#If fRichText Then
Property Get SelStrikethru() As Boolean
    SelStrikethru = rtb.SelStrikethru
End Property

Property Let SelStrikethru(fSelStrikethruA As Boolean)
    rtb.SelStrikethru = fSelStrikethruA
End Property

Property Get SelTabCount() As Integer
    SelTabCount = rtb.SelTabCount
End Property

Property Let SelTabCount(iSelTabCountA As Integer)
    rtb.SelTabCount = iSelTabCountA
End Property

Property Get SelTabs(i As Integer) As Single
    SelTabs = rtb.SelTabs(i)
End Property

Property Let SelTabs(i As Integer, rSelTabLocA As Single)
    rtb.SelTabs(i) = rSelTabLocA
End Property
#End If

Property Get SelText() As String
    SelText = rtb.SelText
End Property

Property Let SelText(sSelTextA As String)
    rtb.SelText = sSelTextA
End Property

#If fRichText Then
Property Get SelUnderline() As Boolean
    SelUnderline = rtb.SelUnderline
End Property

Property Let SelUnderline(fSelUnderlineA As Boolean)
    rtb.SelUnderline = fSelUnderlineA
End Property
#End If

Property Get TabIndex() As Integer
    TabIndex = rtb.TabIndex
End Property

Property Let TabIndex(iTabIndexA As Integer)
    rtb.TabIndex = iTabIndexA
End Property

Property Get TabStop() As Boolean
    TabStop = rtb.TabStop
End Property

Property Let TabStop(fTabStopA As Boolean)
    rtb.TabStop = fTabStopA
End Property

Property Get Tag() As String
    Tag = rtb.Tag
End Property

Property Let Tag(sTagA As String)
    rtb.Tag = sTagA
End Property

Property Get Text() As String
    Text = rtb.Text
End Property

Property Let Text(sTextA As String)
    rtb.Text = sTextA
End Property

#If fRichText Then
Property Get TextRTF() As String
    TextRTF = rtb.TextRTF
End Property

Property Let TextRTF(sTextRTFA As String)
    rtb.TextRTF = sTextRTFA
End Property
#End If

Property Get Top() As Single
    Top = rtb.Top
End Property

Property Let Top(rTopA As Single)
    rtb.Top = rTopA
End Property

Property Get Visible() As Boolean
    Visible = rtb.Visible
End Property

Property Let Visible(fVisibleA As Boolean)
    rtb.Visible = fVisibleA
End Property

Property Get WhatsThisHelpID() As Long
    WhatsThisHelpID = rtb.WhatsThisHelpID
End Property

Property Let WhatsThisHelpID(iWhatsThisHelpIDA As Long)
    rtb.WhatsThisHelpID = iWhatsThisHelpIDA
End Property

Property Get Width() As Single
    Width = rtb.Width
End Property

Property Let Width(rWidthA As Single)
    rtb.Width = rWidthA
End Property

'' Methods passed through

Public Sub Drag(Optional ordAction As Variant)
    If IsMissing(ordAction) Then ordAction = vbBeginDrag
    rtb.Drag ordAction
End Sub

Function find(sSearch As String, Optional vStart As Variant, _
              Optional vEnd As Variant, Optional vOptions As Variant) As Long
With rtb
#If fRichText Then
    If IsMissing(vStart) Then
        If IsMissing(vEnd) Then
            ' Both missing
            If rtb.SelLength > 0 Then
                vStart = .SelStart
                vEnd = .SelStart + .SelLength
            Else
                ' Enhance to start at current position
                vStart = .SelStart
                vEnd = .SelStart - 1
            End If
        Else
            ' Start missing
            vStart = .SelStart
        End If
    Else
        If IsMissing(vEnd) Then
            ' End missing
            vEnd = Characters
        ' else
            ' None missing
        End If
    End If
    If IsMissing(vOptions) Then vOptions = 0
    .find sSearch, vStart, vEnd, vOptions
#End If
End With
End Function

#If fRichText = 0 Then
Public Sub LinkExecute(sCmd As String)
    rtb.LinkExecute sCmd
End Sub

Public Sub LinkPoke()
    rtb.LinkPoke
End Sub

Public Sub LinkSend()
    rtb.LinkSend
End Sub
#End If

''@B LoadFile
Sub LoadFile(sFileName As String, Optional vFileType As Variant)
With rtb
    If IsMissing(vFileType) Then vFileType = rtfRTF
' Late discovered incompatibility means you can't load a text file
' with CR/LF in them because these are considered "binary" characters.
' So text load files the old way.
#If fRichTextBox Then
    If vFileType = rtfRTF Then
        ' Pass through RichTextBox method
        .LoadFile sFileName, vFileType
        sFileCur = sFileName
        DirtyBit = False
        Exit Sub
    End If
#End If
    ' Ignore optional argument
    Dim s As String
    s = GetFileText(sFileName, errLast)
    If errLast = 0 Then
        If Len(s) <> FileLen(sFileName) Then
            ' Use this to mean file too long
            errLast = 62 ' Input past end of file
        Else
            rtb.Text = s
            sFileCur = sFileName
        End If
    End If
    DirtyBit = False
End With
End Sub
''@E LoadFile

''@B Move
Sub Move(X As Variant, Optional Y As Variant, _
         Optional dx As Variant, Optional dy As Variant)
    If IsMissing(Y) Then
        rtb.Move X
    ElseIf IsMissing(dx) Then
        rtb.Move X, Y
    ElseIf IsMissing(dy) Then
        rtb.Move X, Y, dx
    Else
        rtb.Move X, Y, dx, dy
    End If
End Sub
''@E Move

''@B Refresh
Public Sub Refresh()
    rtb.Refresh
End Sub
''@E Refresh

Sub SaveFile(sFileName As String, Optional vFileType As Variant)
With rtb
#If fRichText Then
    If IsMissing(vFileType) Then vFileType = 0
    .SaveFile sFileName, vFileType
#Else
    Dim e As Long
    e = SaveFileText(sFileName, .Text)
    If e <> 0 Then Error e
#End If
End With
End Sub

Public Sub SelPrint(hDC) ' (As SysInt)
With rtb
#If fRichText Then
    .SelPrint hDC
#Else
    Dim s As String, prt As Printer
    For Each prt In Printers
        If prt.hDC = hDC Then Exit For
    Next
    If .SelLength > 0 Then
        s = .SelText
    Else
        s = .Text
    End If
    prt.Print s
#End If
End With
End Sub

Public Sub SetFocus()
    rtb.SetFocus
End Sub

Public Sub ShowWhatsThis()
    rtb.ShowWhatsThis
End Sub

Public Sub Span(sCharSet As String, Optional vForward As Variant, _
                Optional vNegate As Variant)
    If IsMissing(vForward) Then vForward = True
    If IsMissing(vNegate) Then vNegate = False
#If fRichText Then
    rtb.Span sCharSet, vForward, vNegate
#Else
    BugMessage "Exercise for the reader"
#End If
End Sub

Public Sub UpTo(sCharSet As String, Optional vForward As Variant, _
                Optional vNegate As Variant)
    If IsMissing(vForward) Then vForward = True
    If IsMissing(vNegate) Then vNegate = False
#If fRichText Then
    rtb.UpTo sCharSet, vForward, vNegate
#Else
    BugMessage "Exercise for the reader"
#End If
End Sub

Public Sub ZOrder(Optional iPosition As Variant)
    If IsMissing(iPosition) Then iPosition = 0
    rtb.ZOrder iPosition
End Sub

''@B KeyPress
Public Sub KeyPress(KeyAscii As Integer)
    If KeyAscii > 31 Then
        If OverWrite Then
            rtb.SelLength = rtb.SelLength + 1
            ' Don't overwrite LF in CR/LF sequence
            If Right$(rtb.SelText, 1) = sCr Then
                rtb.SelLength = rtb.SelLength - 1
            End If
        End If
    End If
End Sub
''@E KeyPress

Public Sub KeyUp(KeyCode As Integer, Shift As Integer)
    'If KeyCode = vbKeyDelete Then
    '    If rtb.SelText = sEmpty Then rtb.SelLength = 1
    '    rtb.SelText = sEmpty
    'End If
End Sub

''@B Lo
#If Win32 Then
Private Function Lo(ByVal i As Long) As Long
    Lo = i
End Function
#Else
Private Function Lo(ByVal i As Long) As Integer
    Lo = LoWord(i)
End Function
#End If
''@E Lo
'
