VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CFileInfo"
Attribute VB_Creatable = True
Attribute VB_Exposed = False
Option Explicit
#If Win32 Then
DefLng A-Z
#Else
DefInt A-Z
#End If

Private vFileItem As Variant
Private fi As TFileInfo
Private fd As TFindData
Private fCreated As Boolean
Private afAttr As Long
Private afOption As Long
Private vModified As Variant
Private vAccessed As Variant
Private vCreated As Variant

''@B Create
Function Create(vFileItemA As Variant) As Boolean
    Dim f As Boolean, h As Long
    Destroy     ' Assume fail
    If VarType(vFileItemA) = vbString Then
        If Len(vFileItemA) <= 3 And Mid$(vFileItemA, 2, 1) = ":" Then
            ' Must be drive, get attributes
            afAttr = 0: afOption = 0
        Else
            ' For file, get information in advance
            h = FindFirstFile(vFileItemA, fd)
            If h = -1 Then Exit Function
            FindClose h
            afAttr = fd.dwFileAttributes
            afOption = SHGFI_USEFILEATTRIBUTES
        End If
        f = GetFileInfo(CStr(vFileItemA), fi, afOption Or _
                        SHGFI_DISPLAYNAME Or SHGFI_TYPENAME, afAttr)
    Else
        ' For special folders, everything comes from system
        afAttr = 0: afOption = 0
        f = GetItemInfo(CLng(vFileItemA), fi, _
                        SHGFI_DISPLAYNAME Or SHGFI_TYPENAME)
    End If
    If f Then
        vFileItem = vFileItemA
        fCreated = True
    End If
    Create = fCreated
End Function
''@E Create

Function CreateFile(sFile As String, afAttrA As Long, cLen As Long, _
                    vModifiedA As Variant, vAccessedA As Variant, _
                    vCreatedA As Variant) As Boolean
    Dim f As Long
    Destroy
    afAttr = afAttrA
    vModified = vModifiedA
    vAccessed = vAccessedA
    vCreated = vCreatedA
    fd.nFileSizeLow = cLen
    fd.nFileSizeHigh = 0
    f = GetFileInfo(sFile, fi, SHGFI_DISPLAYNAME Or SHGFI_TYPENAME _
                               Or SHGFI_ATTRIBUTES, afAttr)
    If f Then
        vFileItem = sFile
        fCreated = True
        afAttr = fi.dwAttributes
    End If
    CreateFile = fCreated
End Function

Private Sub Destroy()
    fCreated = False
    vFileItem = Empty
    vModified = Empty
    vAccessed = Empty
    vCreated = Empty
End Sub

Property Get DisplayName() As String
    If fCreated Then DisplayName = StrZToStr(fi.szDisplayName)
    If DisplayName = sEmpty Then DisplayName = "Unknown"
End Property

' Returns file type
Property Get TypeName() As String
    If fCreated Then TypeName = StrZToStr(fi.szTypeName)
End Property

' Returns type of executable file as string, and optionally returns
' executable type and version through reference variables:
'
'    Return   = Type string
'    vExeType = fiWin16Exe, fiDosExe, fiWin32Exe, fiWin32Console
'    vExeVer  = (3.0, 3.1, 3.5, 4.0, etc.)
Function ExeType(Optional vWinType As Variant, _
                 Optional vWinVer As Variant) As String
    If fCreated = False Or VarType(vFileItem) <> vbString Then Exit Function
    Dim dw As Long, w As Integer, fiT As TFileInfo
    
    dw = GetFileItemInfo(vFileItem, fiT, afOption Or SHGFI_EXETYPE, afAttr)
    w = HiWord(dw)
    vWinType = LoWord(dw)
    vWinVer = HiByte(w) + CSng(LoByte(w) / 100)
    
    Select Case vWinType
    Case fiWin32Exe
        Select Case vWinVer
        Case 0
            vWinType = fiWin32Console
            ExeType = "Win32 Console Application"
        Case Is >= 4#
            ExeType = "Windows 95 Application"
        Case Is > 3.1
            ExeType = "Windows NT Application"
        Case 3.1
            ExeType = "Win32S Application"
        End Select
    Case fiWin16Exe
        ExeType = "Windows 3.x Application"
    Case fiDosExe
        ExeType = "MS-DOS Application or Batch File"
    Case 0
        ExeType = sEmpty    ' Not an executable
    Case Else
        ExeType = "Unknown Executable"
    End Select
End Function

''@B SmallIcon
Function SmallIcon(Optional vOverlay As Variant) As Picture
    Dim fiT As TFileInfo
    If fCreated = False Then Exit Function
    If IsMissing(vOverlay) Then vOverlay = 0&
    ' Filter out any invalid flags -- only overlays allowed
    vOverlay = vOverlay And (SHGFI_LINKOVERLAY Or SHGFI_SELECTED _
                             Or SHGFI_OPENICON)
    ' Add in standard and small icon flags
    vOverlay = vOverlay Or afOption Or SHGFI_ICON Or SHGFI_SMALLICON
    GetFileItemInfo vFileItem, fiT, vOverlay, afAttr
    Set SmallIcon = IconToPicture(fiT.hIcon)
End Function
''@E SmallIcon

Function LargeIcon(Optional vOverlay As Variant) As Picture
    Dim fiT As TFileInfo
    If fCreated = False Then Exit Function
    If IsMissing(vOverlay) Then vOverlay = 0&
    ' Filter out any invalid flags -- only overlays allowed
    vOverlay = vOverlay And (SHGFI_LINKOVERLAY Or SHGFI_SELECTED _
                             Or SHGFI_OPENICON)
    ' Add in standard and large icon flags
    vOverlay = vOverlay Or afOption Or SHGFI_ICON Or SHGFI_LARGEICON
    GetFileItemInfo vFileItem, fiT, vOverlay, afAttr
    Set LargeIcon = IconToPicture(fiT.hIcon)
End Function

Function ShellIcon(Optional vOverlay As Variant) As Picture
    Dim fiT As TFileInfo
    If fCreated = False Then Exit Function
    If IsMissing(vOverlay) Then vOverlay = 0&
    ' Filter out any invalid flags -- only overlays allowed
    vOverlay = vOverlay And (SHGFI_LINKOVERLAY Or SHGFI_SELECTED _
                             Or SHGFI_OPENICON)
    ' Add in standard and large icon flags
    vOverlay = vOverlay Or afOption Or SHGFI_ICON Or SHGFI_SHELLICONSIZE
    GetFileItemInfo vFileItem, fiT, vOverlay, afAttr
    Set ShellIcon = IconToPicture(fiT.hIcon)
End Function

Function Icon(afKind As Long) As Picture
    Dim fiT As TFileInfo
    If fCreated = False Then Exit Function
    GetFileItemInfo vFileItem, fiT, afOption Or SHGFI_ICON Or afKind, afAttr
    Set Icon = IconToPicture(fiT.hIcon)
End Function

' Retrieves file attribute flags:
'       ReadOnly        Hidden      System      Directory
'       Archive         Normal      Temporary   Compressed
Function Attributes() As Long
    If fCreated = False Or vFileItem = sEmpty Then Exit Function
    Attributes = fd.dwFileAttributes
End Function

Function Length() As Long
    If fCreated Then
        Length = fd.nFileSizeLow
        If fd.nFileSizeHigh Then Length = -1
    End If
End Function

Function Modified() As Variant
    If fCreated = False Or VarType(vFileItem) <> vbString Then Exit Function
    If vModified = Empty Then vModified = SysToVBTime(fd.ftLastWriteTime)
    Modified = vModified
End Function

Function Created() As Variant
    If fCreated = False Or VarType(vFileItem) <> vbString Then Exit Function
    If vCreated = Empty Then vCreated = SysToVBTime(fd.ftCreationTime)
    Created = vCreated
End Function

Function Accessed() As Variant
    If fCreated = False Or VarType(vFileItem) <> vbString Then Exit Function
    If vAccessed = Empty Then vAccessed = SysToVBTime(fd.ftLastAccessTime)
    Accessed = vAccessed
End Function

