VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CPictureGlass"
Attribute VB_Creatable = True
Attribute VB_Exposed = False
Option Explicit
#If Win32 Then
DefLng A-Z
#Else
DefInt A-Z
#End If

''@B PGInternal
Private rpb As PictureBox    ' Internal reference to PictureBox
Private hdcImage, hbmpImage, hbmpImageOld ' As SysInt
Private hdcMask, hbmpMask, hbmpMaskOld ' As SysInt
Private hdcBack, hbmpBack, hbmpBackOld ' As SysInt
Private hdcCache, hbmpCache, hbmpCacheOld ' As SysInt
Private fExist As Boolean, fVisible As Boolean
Private xOld, yOld, dxBlt, dyBlt ' As SysInt
Private rop As Long
''@E PGInternal

''@B Create1
Function Create(rpbA As PictureBox) As Boolean
    
    ' Clean up any old instance before creating a new one
    If fExist Then Destroy
    On Error GoTo CreateError
    Set rpb = rpbA
    With rpb
        ' Picture must be invisible and borderless with AutoDraw on
        .Visible = False
        .BorderStyle = vbBSNone: .AutoRedraw = True
        ' Get size of image for copies
        dxBlt = .ScaleX(.Width, .ScaleMode, vbPixels)
        dyBlt = .ScaleY(.Height, .ScaleMode, vbPixels)
    End With
''@E Create1
    
''@B Create2
    ' Create memory DC for image with inverted background (AND mask)
    ' Create DC compatible with screen
    hdcImage = CreateCompatibleDC(0&)
    Dim cPlanes, cPixelBits ' As SysInt
    ''@B GetDeviceCaps
    cPlanes = GetDeviceCaps(hdcImage, PLANES)
    cPixelBits = GetDeviceCaps(hdcImage, BITSPIXEL)
    ''@E GetDeviceCaps
    ' Create color bitmap same as screen
    hbmpImage = CreateBitmap(dxBlt, dyBlt, cPlanes, cPixelBits, 0&)
    hbmpImageOld = SelectObject(hdcImage, hbmpImage)
    ' Copy image (we'll invert its background later)
    Call BitBlt(hdcImage, 0, 0, dxBlt, dyBlt, rpb.hDC, 0, 0, vbSrcCopy)
''@E Create2
    
''@B Create3
    ' Create DC for monochrome mask of image (XOR mask)
    ' Create device context compatible with screen
    hdcMask = CreateCompatibleDC(0&)
    ' Create bitmap (monochrome by default)
    hbmpMask = CreateCompatibleBitmap(hdcMask, dxBlt, dyBlt)
    ' Select it into DC
    hbmpMaskOld = SelectObject(hdcMask, hbmpMask)
    ' Copy color bitmap to monochrome DC to create mono mask
    Call BitBlt(hdcMask, 0, 0, dxBlt, dyBlt, hdcImage, 0, 0, vbSrcCopy)
''@E Create3
    
''@B Create4
    ' Invert background of image to create AND Mask
    Call SetBkColor(hdcImage, vbBlack)
    Call SetTextColor(hdcImage, vbWhite)
    Call BitBlt(hdcImage, 0, 0, dxBlt, dyBlt, hdcMask, 0, 0, vbSrcAnd)
''@E Create4
                
''@B Create5
    ' Create memory DCs for old background and cache
    hdcBack = CreateCompatibleDC(0&)
    hbmpBack = CreateBitmap(dxBlt, dyBlt, cPlanes, cPixelBits, 0&)
    hbmpBackOld = SelectObject(hdcBack, hbmpBack)
    hdcCache = CreateCompatibleDC(0&)
    hbmpCache = CreateBitmap(dxBlt, dyBlt, cPlanes, cPixelBits, 0&)
    hbmpCacheOld = SelectObject(hdcCache, hbmpCache)
    
    ' Invalid x and y indicate first move hasn't occurred
    xOld = -1: yOld = -1
    fExist = True: fVisible = True
    Create = True
    Exit Function
CreateError:
    ' Create = False
End Function
''@E Create5

''@B Class_Terminate
Private Sub Class_Terminate()
    Destroy
End Sub
''@E Class_Terminate

''@B Destroy
Sub Destroy()
    ' Select old mask back to DC
    Call SelectObject(hdcMask, hbmpMaskOld)
    ' Now it's safe to delete DC and bitmask
    Call DeleteDC(hdcMask)
    Call DeleteObject(hbmpMask)
    ' Clean up inverted image DC
    Call SelectObject(hdcImage, hbmpImageOld)
    Call DeleteDC(hdcImage)
    Call DeleteObject(hbmpImage)
    ' Clean up cache DC
    Call SelectObject(hdcCache, hbmpCacheOld)
    Call DeleteDC(hdcCache)
    Call DeleteObject(hbmpCache)
    ' Clean up old background DC
    Call SelectObject(hdcBack, hbmpBackOld)
    Call DeleteDC(hdcBack)
    Call DeleteObject(hbmpBack)
    ' Turn off picture box
    Set rpb = Nothing
    fExist = False
End Sub
''@E Destroy

''@B Draw
Public Sub Draw()
With rpb.Container
    BugAssert fExist
    If fVisible = False Then Exit Sub
    ' Calculate background position on its container
    Dim fAutoRedraw As Boolean, xDst, yDst ' As SysInt
    xDst = .ScaleX(rpb.Left, .ScaleMode, vbPixels)
    yDst = .ScaleY(rpb.Top, .ScaleMode, vbPixels)
    fAutoRedraw = .AutoRedraw
    .AutoRedraw = False
    Dim clrBack As Long, clrFore As Long, hdcDst ' As SysInt
    hdcDst = .hDC
    
    ' Copy old background to its last location
    If xOld <> -1 Then
        Call BitBlt(hdcDst, xOld, yOld, dxBlt, dyBlt, _
                    hdcBack, 0, 0, vbSrcCopy)
    End If
    ' Save current background and position for next time
    Call BitBlt(hdcBack, 0, 0, dxBlt, dyBlt, _
                hdcDst, xDst, yDst, vbSrcCopy)
    ' Create cache copy of background to work on
    Call BitBlt(hdcCache, 0, 0, dxBlt, dyBlt, _
                hdcDst, xDst, yDst, vbSrcCopy)
    xOld = xDst: yOld = yDst
    ' Save color and set to black and white
    clrBack = GetBkColor(hdcCache)
    clrFore = GetTextColor(hdcCache)
    Call SetBkColor(hdcCache, vbWhite)
    Call SetTextColor(hdcCache, vbBlack)
    ' Mask the background
    Call BitBlt(hdcCache, 0, 0, dxBlt, dyBlt, _
                hdcMask, 0, 0, vbSrcAnd)
    ' Put image in hole created by mask
    Call BitBlt(hdcCache, 0, 0, dxBlt, dyBlt, _
                hdcImage, 0, 0, vbSrcPaint)
    ' Restore color
    Call SetBkColor(hdcCache, clrBack)
    Call SetTextColor(hdcCache, clrFore)
    ' Put finished cache on screen
    Call BitBlt(hdcDst, xDst, yDst, dxBlt, dyBlt, _
                hdcCache, 0, 0, vbSrcCopy)
    .AutoRedraw = fAutoRedraw
End With
End Sub
''@E Draw

Public Sub Remove()
With rpb.Container
    BugAssert fExist
    If fVisible = False Then Exit Sub
        
    ' Calculate the background position on its form
    Dim fAutoRedraw As Boolean, xDst, yDst ' As SysInt
    xDst = .ScaleX(rpb.Left, .ScaleMode, vbPixels)
    yDst = .ScaleY(rpb.Top, .ScaleMode, vbPixels)
    fAutoRedraw = .Container.AutoRedraw
    .AutoRedraw = False
    
    ' Copy the old background to its last location
    If xOld <> -1 Then
        Call BitBlt(.hDC, xOld, yOld, dxBlt, dyBlt, _
                    hdcBack, 0, 0, vbSrcCopy)
    End If
    .AutoRedraw = fAutoRedraw
End With
End Sub

''@B Move
Public Sub Move(xLeft As Variant, Optional yTop As Variant)
    BugAssert fExist
    If IsMissing(yTop) Then
        rpb.Move xLeft
    Else
        rpb.Move xLeft, yTop
    End If
    Draw
End Sub
''@E Move

Property Get BackColor() As Long
    BugAssert fExist
    BackColor = rpb.BackColor
End Property

Property Get ForeColor() As Long
    BugAssert fExist
    ForeColor = rpb.ForeColor
End Property

''@B GetLeft
Property Get Left() As Single
    BugAssert fExist
    Left = rpb.Left
End Property
''@E GetLeft

Property Let Left(rLeftA As Single)
    BugAssert fExist
    rpb.Left = rLeftA
    Draw
End Property

Property Get Top() As Single
    BugAssert fExist
    Top = rpb.Top
End Property

Property Let Top(rTopA As Single)
    BugAssert fExist
    rpb.Top = rTopA
    Draw
End Property

Property Get Width() As Single
    BugAssert fExist
    Width = rpb.Width
End Property

Property Get Height() As Single
    BugAssert fExist
    Height = rpb.Height
End Property

''@B Visible
Property Get Visible() As Boolean
    BugAssert fExist
    Visible = fVisible
End Property

Property Let Visible(fVisibleA As Boolean)
    BugAssert fExist
    fVisible = fVisibleA
End Property
''@E Visible
'

