Attribute VB_Name = "MRegTypeLib"
Option Explicit

''@B LoadTypeLib
#If Win32 Then
Private Declare Function LoadTypeLib Lib "oleaut32.dll" ( _
    pFileName As Byte, pobjTLib As Object) As Long
#Else
Private Declare Function LoadTypeLib Lib "typelib.dll" ( _
    pFileName As Byte, pobjTLib As Object) As Long
#End If
''@E LoadTypeLib
#If Win32 Then
Private Declare Function RegisterTypeLib Lib "oleaut32.dll" ( _
    ByVal objTLib As Object, pFileName As Byte, _
    pHelpFile As Byte) As Long
#Else
Private Declare Function RegisterTypeLib Lib "typelib.dll" ( _
    ByVal objTLib As Object, pFileName As Byte, _
    pHelpFile As Byte) As Long
#End If

Public Const sEmpty = ""
Public Const sQuote2 = """"

Sub Main()
    Dim fSilent As Boolean, fVerbose As Boolean
    Dim sCmd As String, i As Integer, errOK As Long
    Dim sSep As String, sToken As String, sLib As String
    sCmd = Command$
    If sCmd = sEmpty Then
        sCmd = InputBox("Enter type library name: ")
    End If
    sSep = " " & Chr$(9)
    
    ' Parse command line
    sToken = GetToken(sCmd, sSep)
    Do While sToken <> sEmpty
        If InStr("/-", Left$(sToken, 1)) Then
            Select Case UCase$(Mid$(sToken, 2, 1))
            Case "S"
                fSilent = True
            Case "V"
                fVerbose = True
            Case Else
                ShowSyntax "Unknown option", fSilent
                End
            End Select
        Else
            sLib = sToken
        End If
        sToken = GetToken(sEmpty, sSep)
    Loop
    
    Dim sExt As String
    Dim sBase As String, sBase16 As String, sBase32 As String
    Dim sFull As String, sFull16 As String, sFull32 As String
    Dim iDir As Long, iBase As Long, iExt As Long
    ' Validate extension
    iDir = GetExtPos(sLib)
    sExt = Mid$(sLib, iDir)
    sBase = Left$(sLib, iDir - 1)
    Select Case UCase$(sExt)
    Case sEmpty
        ShowSyntax "No extension given", fSilent
        End
    Case ".TLB", ".OLB", ".DLL"
    Case Else
        ShowSyntax "Unknown extension", fSilent
        End
    End Select
    ' Attempt to register three possible names
    If Right$(sBase, 2) = "32" Then
        sBase32 = sBase
        sBase = Left$(sBase, Len(sBase) - 2)
        sBase16 = sBase & "16"
    ElseIf Right$(sBase, 2) = "16" Then
        sBase16 = sBase
        sBase = Left$(sBase, Len(sBase) - 2)
        sBase32 = sBase & "32"
    Else
        sBase32 = sBase & "32"
        sBase16 = sBase & "16"
    End If
        
    sFull = SearchDirs(sEmpty, sBase & sExt, sEmpty, _
                       iDir, iBase, iExt)
    sFull16 = SearchDirs(sEmpty, sBase16 & sExt, sEmpty, _
                         iDir, iBase, iExt)
    sFull32 = SearchDirs(sEmpty, sBase32 & sExt, sEmpty, _
                         iDir, iBase, iExt)
    ' Register full name if given, or try to create 16/32 names
    sLib = sEmpty
    If sFull <> sEmpty Then
        errOK = RegTypelib(sFull)
        If errOK Then sLib = sFull
    Else
        If sFull16 = sEmpty And sFull32 = sEmpty Then
            ShowSyntax "File not found", fSilent
            End
        End If
#If Win32 Then
        errOK = RegTypelib(sFull16)
        If errOK Then sLib = sFull16
        errOK = RegTypelib(sFull32)
        If errOK Then sLib = sLib & " " & sFull32
#Else
        errOK = RegTypelib(sFull32)
        If errOK Then sLib = sFull32
        errOK = RegTypelib(sFull16)
        If errOK Then sLib = sLib & " " & sFull16
#End If
    End If
    If errOK Then
        If Not fSilent Then MsgBox "Can't register type library: " & sLib
    Else
        If fVerbose Then MsgBox "Type library registered: " & sLib
    End If
    
End Sub

''@B RegTypeLib
Function RegTypelib(sLib As String) As Long
    Dim suLib() As Byte, errOK As Long, objTLib As Object
    suLib = sLib & vbNullChar
    errOK = LoadTypeLib(suLib(0), objTLib)
    If errOK = 0 Then errOK = RegisterTypeLib(objTLib, suLib(0), 0&)
    Set objTLib = Nothing
    RegTypelib = errOK
End Function
''@E RegTypeLib

Function GetExtPos(sFile As String) As Integer
    Dim i As Integer
    For i = Len(sFile) To 1 Step -1
        If Mid$(sFile, i, 1) = "." Then
            GetExtPos = i
            Exit Function
        End If
    Next
    GetExtPos = Len(sFile) + 1
End Function

Sub ShowSyntax(sErr As String, fSilent As Boolean)
    If fSilent Then Exit Sub
    Dim sMsg As String
#If Win32 Then
    Const sProg = "REGTLB32"
#Else
    Const sProg = "REGTLB16"
#End If
    sMsg = sErr & sCr & sCr
    sMsg = sMsg & _
        sTab & "Syntax: " & sProg & " [/s] libname.ext" & sCr & sCr & _
        sTab & "/s - Silent (don't show this message box)" & sCr & _
        sTab & "/v - Verbose (report success)" & sCr & sCr
    sMsg = sMsg & sProg & _
        " will attempt to register both 16-bit and 32-bit libraries." & sCr & _
        "For example, to register WIN16.TLB and WIN32.TLB, give any " & sCr & _
        "of these commands: " & sCr & sCr
    sMsg = sMsg & sTab & sProg & " WIN.TLB" & sCr
    sMsg = sMsg & sTab & sProg & " WIN32.TLB" & sCr
    sMsg = sMsg & sTab & sProg & " WIN16.TLB" & sCr
    MsgBox sMsg
End Sub

' Duplicated from BYTES.BAS so we don't use whole module
Function BytesToStr(ab() As Byte) As String
#If Win32 Then
    BytesToStr = StrConv(ab(), vbUnicode)
#Else
    BytesToStr = ab()
#End If
End Function

