// ERRINFO.CPP
//
// Standard error support class to handle project-specific errors.
//

#include "stdhdr.h"
#include "srvdata.h"
#include "ErrInfo.h"

// Constructor and destructor
CSupportErrorInfo::CSupportErrorInfo()
{
	m_refs = 0;
    m_Desc = SysAllocStringLen(NULL, 0);
	m_pcerrinfo = NULL;
}

CSupportErrorInfo::~CSupportErrorInfo()
{
	if (m_Desc) {
		SysFreeString(m_Desc);
	}
	if (m_pcerrinfo) {
		m_pcerrinfo->Release();
	}
}                   

// Standard IUnknown methods
STDMETHODIMP
CSupportErrorInfo::QueryInterface(REFIID riid, void FAR * FAR * ppvObj)
{                                                                     
	if (IIDOK(IUnknown) || IIDOK(ISupportErrorInfo)) {
		*ppvObj = this;
	} else {
		*ppvObj = NULL;
		return ResultFromScode(E_NOINTERFACE);
	}
	AddRef();
	return NOERROR;
}

STDMETHODIMP_(ULONG)
CSupportErrorInfo::AddRef(void)
{
    return ++m_refs;
}

STDMETHODIMP_(ULONG)
CSupportErrorInfo::Release(void)
{
	if (--m_refs == 0) {
        delete this;
        return 0;
    }
    return m_refs;
}

STDMETHODIMP
CSupportErrorInfo::InterfaceSupportsErrorInfo(REFIID riid)
{
	HRESULT hr;
	IErrorInfo FAR * perrinfo;

	if (!m_pcerrinfo) {
		if ((hr = CreateErrorInfo(&m_pcerrinfo)) != NOERROR) {
			m_pcerrinfo = NULL;
			return hr;
		}
		m_pcerrinfo->SetSource(szoLib);
	}
	m_pcerrinfo->SetDescription(m_Desc);

	// If we get this far, this will always work (no error check needed)
	m_pcerrinfo->QueryInterface(IID_IErrorInfo,
								(void FAR * FAR *) &perrinfo);
	hr = SetErrorInfo(0, perrinfo);
	perrinfo->Release();
	return hr;
}
