// HOSTENV.H
//
// Conversion between ANSI and Unicode strings.
//

#ifndef _HOSTENV_
#define _HOSTENV_


/*

The following macros can be used in order to use the same code for
WIN16, WIN32/OAU/UNICODE and WIN32/OAU preprocessor definitions.
Strings are declared with the correct type (Wide or MultiByte) and
are converted correctly to support both Windows and OLE2.0 API calls.

Macros:

OLECHAR: A character compatible with OLE API's
LPOLESTR: A pointer to an string used by OLE API's
OLESTR(x): A constant string for use by OLE API's, eg OLESTR("foo")
SYSCHAR: A character compatible with Windows API's
LPSYSSTR: A pointer to an string used by Windows API's
SYSSTR(x): A constant string for use by Windows API's, eg SYSSTR("foo")
SYSOLESTR(x): Convert OLESTR to SYSSTR to use with system API calls
OLESYSSTR(x): Convert SYSSTR to OLESTR to use with OLE API calls
FREESTR(x): Free temp string created by OLESYSSTR or SYSOLESTR

Example use of macros:

	// Allocate a string buffer for a system call
	SYSCHAR szSysFileName[128];
	// A string pointer for use in OLE calls
	LPOLESTR lpOleFileName;
	// A Windows API call
	GetModuleFileName(g_hinst, szSysFileName, 128);
	// Allocate a new converted string if necessary, copy the pointer if not
	lpOleFileName = OLESYSSTR(szSysFileName);
	// An OLE API call
	LoadTypeLib(lpOleFileName, &ptlib);
	// Free string allocated by OLESYSSTR
	FREESTR(lpOleFileName);

*/

#ifdef WIN32
#define OLECHAR   WCHAR
#define LPOLESTR  LPWSTR
#define OLESTR(x) L##x
#else   
#define OLECHAR   char
#define LPOLESTR  LPSTR
#define OLESTR(x) x
#endif  


#ifdef UNICODE
#define SYSCHAR     WCHAR
#define LPSYSSTR    LPWSTR
#define SYSSTR(x)   L##x
#define SYSOLESTR(x) x
#define OLESYSSTR(x) x
#define FREESTR(x)
#else
#define SYSCHAR     char
#define LPSYSSTR    LPSTR
#define SYSSTR(x)   x

#ifdef WIN32
// Implemented in HOSTENV.CPP
LPSYSSTR UNICODEtoANSI(BSTR bstrUNICODE);
BSTR ANSItoUNICODE(LPSYSSTR bstrANSI);
#define SYSOLESTR(x) UNICODEtoANSI(x)
#define OLESYSSTR(x) ANSItoUNICODE(x)
#define FREESTR(x)   SysFreeString((LPOLESTR)x)
#else
#define SYSOLESTR(x) x
#define OLESYSSTR(x) x
#define FREESTR(x)
#endif // WIN32
#endif // UNICODE
				   
#endif // _HOSTENV_
