Attribute VB_Name = "MSort"
Option Explicit
#If Win32 Then
DefLng A-Z
#Else
DefInt A-Z
#End If

'$ Uses UTILITY.BAS

''@B SortArray
' QuickSort algorithm
Sub SortArray(aTarget() As Variant, _
              iFirst As Integer, iLast As Integer)
    Dim vSplit As Variant
 
    If iFirst < iLast Then

        ' Only two elements in this subdivision; exchange if
        ' they are out of order, and end recursive calls
        If iLast - iFirst = 1 Then
            ''@B CallSortCompare
            If SortCompare(aTarget(iFirst), aTarget(iLast)) > 0 Then
                SortSwap aTarget(iFirst), aTarget(iLast)
            End If
            ''@E CallSortCompare
        Else

            Dim i As Integer, j As Integer, iRand As Integer

            ' Pick pivot element at random and move to end
            ' (consider calling Randomize before sorting)
            iRand = GetRandom(iFirst, iLast)
            SortSwap aTarget(iLast), aTarget(iRand)
            vSplit = aTarget(iLast)
            Do

                ' Move in from both sides toward pivot element
                i = iFirst: j = iLast
                Do While (i < j) And _
                         SortCompare(aTarget(i), vSplit) <= 0
                    i = i + 1
                Loop
                Do While (j > i) And _
                         SortCompare(aTarget(j), vSplit) >= 0
                    j = j - 1
                Loop

                ' If you haven't reached pivot element, it means
                ' that the two elements on either side are out of
                ' order, so swap them
                If i < j Then
                    SortSwap aTarget(i), aTarget(j)
                End If
            Loop While i < j

            ' Move pivot element back to its proper place
            SortSwap aTarget(i), aTarget(iLast)

            ' Recursively call SortArray (pass smaller
            ' subdivision first to use less stack space)
            If (i - iFirst) < (iLast - i) Then
                SortArray aTarget(), iFirst, i - 1
                SortArray aTarget(), i + 1, iLast
            Else
                SortArray aTarget(), i + 1, iLast
                SortArray aTarget(), iFirst, i - 1
            End If
        End If
    End If

End Sub
''@E SortArray

' QuickSort algorithm
''@B SortCollection
Sub SortCollection(nTarget As Collection, iFirst As Integer, iLast As Integer)
    Dim vSplit As Variant
    Static fRand As Integer
    If fRand = False Then
        Randomize
        fRand = True
    End If

    If iFirst < iLast Then

        ' Only two elements in this subdivision; exchange if
        ' they are out of order, and end recursive calls
        If iLast - iFirst = 1 Then
            If SortCompare(nTarget(iFirst), nTarget(iLast)) > 0 Then
                CollectionSwap nTarget, iFirst, iLast
            End If
        Else

            Dim i As Integer, j As Integer, iRand As Integer

            ' Pick pivot element at random and move to end
            ' (consider calling Randomize before sorting)
            iRand = GetRandom(iFirst, iLast)
            CollectionSwap nTarget, iLast, iRand
            vSplit = nTarget(iLast)
            Do

                ' Move in from both sides toward pivot element
                i = iFirst: j = iLast
                Do While (i < j) And _
                    SortCompare(nTarget(i), vSplit) <= 0
                    i = i + 1
                Loop
                Do While (j > i) And _
                    SortCompare(nTarget(j), vSplit) >= 0
                    j = j - 1
                Loop

                ' If you haven't reached pivot element, it means
                ' that the two elements on either side are out of
                ' order, so swap them
                If i < j Then
                    CollectionSwap nTarget, i, j
                End If
            Loop While i < j

            ' Move pivot element back to its proper place
            CollectionSwap nTarget, i, iLast

            ' Recursively call SortCollection (pass smaller
            ' subdivision first to use less stack space)
            If (i - iFirst) < (iLast - i) Then
                SortCollection nTarget, iFirst, i - 1
                SortCollection nTarget, i + 1, iLast
            Else
                SortCollection nTarget, i + 1, iLast
                SortCollection nTarget, iFirst, i - 1
            End If
        End If
    End If

End Sub
''@E SortCollection

''@B BSearchArray
Function BSearchArray(av() As Variant, vKey As Variant, _
                      iPos As Integer) As Boolean
    Dim iLo As Integer, iHi As Integer
    Dim iComp As Integer, iMid As Integer
    iLo = LBound(av): iHi = UBound(av)
    Do
        iMid = iLo + ((iHi - iLo) \ 2)
        iComp = SortCompare(av(iMid), vKey)
        Select Case iComp
        Case 0
            ' Item found
            iPos = iMid
            BSearchArray = True
            Exit Function
        Case Is > 0
            ' Item is in upper half
            iHi = iMid
            If iLo = iHi Then Exit Do
        Case Is < 0
            ' Item is in lower half
            iLo = iMid + 1
            If iLo > iHi Then Exit Do
        End Select
    Loop
    ' Item not found, but return position to insert
    iPos = iMid - (iComp < 0)
    BSearchArray = False
        
End Function
''@E BSearchArray

''@B BSearchCollection
Function BSearchCollection(n As Collection, vKey As Variant, _
                           iPos As Integer) As Boolean
    Dim iLo As Integer, iHi As Integer
    Dim iComp As Integer, iMid As Integer
    iLo = 1: iHi = n.Count
    Do
        iMid = iLo + ((iHi - iLo) \ 2)
        iComp = SortCompare(n(iMid), vKey)
        Select Case iComp
        Case 0
            ' Item found
            iPos = iMid
            BSearchCollection = True
            Exit Function
        Case Is > 0
            ' Item is in upper half
            iHi = iMid
            If iLo = iHi Then Exit Do
        Case Is < 0
            ' Item is in lower half
            iLo = iMid + 1
            If iLo > iHi Then Exit Do
        End Select
    Loop
    ' Item not found, but return position to insert
    iPos = iMid - (iComp < 0)
    BSearchCollection = False
        
End Function
''@E BSearchCollection

''@B ShuffleArray
Sub ShuffleArray(a() As Variant)
    Dim iFirst As Integer, iLast As Integer
    iFirst = LBound(a): iLast = UBound(a)
        
    ' Randomize array
    Dim i As Integer, v As Variant, iRnd As Integer
    For i = iLast To iFirst + 1 Step -1
        ' Swap random element with last element
        iRnd = GetRandom(iFirst, i)
        SortSwap a(i), a(iRnd)
    Next
End Sub
''@E ShuffleArray

''@B ShuffleCollection
Sub ShuffleCollection(n As Collection)
    Dim iFirst As Integer, iLast As Integer
    iFirst = 1: iLast = n.Count
    
    ' Randomize collection
    Dim i As Integer, v As Variant, iRnd As Integer
    For i = iLast To iFirst + 1 Step -1
        ' Swap random element with last element
        iRnd = GetRandom(iFirst, i)
        CollectionSwap n, i, iRnd
    Next
End Sub
''@E ShuffleCollection

''@B SortCompare
' Define fSortCompareDef to use default SortCompare
#If fSortCompareDef Then
Private Function SortCompare(ByVal v1 As Variant, _
                             ByVal v2 As Variant) As Integer
    If v1 < v2 Then
        SortCompare = -1
    ElseIf v1 = v2 Then
        SortCompare = 0
    Else
        SortCompare = 1
    End If
End Function
#End If
''@E SortCompare

''@B SortSwap
' Define fSortSwapNoDef if you provide your own swap routine
#If fSortSwapNoDef = 0 Then
Sub SortSwap(v1 As Variant, v2 As Variant)
    Dim vT As Variant
    vT = v1
    v1 = v2
    v2 = vT
End Sub
#End If
''@E SortSwap

''@B CollectionSwap
Sub CollectionSwap(n As Collection, i1 As Integer, i2 As Integer)
    Dim vT As Variant
    vT = n(i1)
    n.Add n(i2), , , i1
    n.Remove i1
    n.Add vT, , , i2
    n.Remove i2
End Sub
''@E CollectionSwap
